/**************************************************************************** 
*  Copyright 2007 C&H Technologies, Inc.  All Rights Reserved.					    * 
*****************************************************************************/
/****************************************************************************
*
* ma204_ansi.h  
*
*	Header file for the MA204 ANSI-C source module.
*
*	C&H Part Number: 11028497
*
*****************************************************************************/
#ifndef	MA204_HEADER
#define	MA204_HEADER

#ifndef PLATFORM
#define		PLATFORM chwin
#endif

#include "apis.h"


/**************************************************************************** 
*  TypeDef's
*****************************************************************************/
typedef char Int8;
typedef unsigned char UInt8;
typedef unsigned short UInt16;
typedef int Int32;
typedef unsigned int UInt32;
typedef unsigned short Bool;
typedef double Real64;


typedef struct
{ Bool	wait_for_ready;
	Real64 wait_timeout;
	APIS_HANDLE apis_handle;
	UInt32	apis_error;
}MA204_INSTR, *MA204_HANDLE;


/**************************************************************************** 
*  Error Codes
*****************************************************************************/
#define		MA204_SUCCESS															0

#define		MA204_SPECIFIC_ERROR_BASE									0xBFFC0900
#define		MA204_ERROR_RESERVED_VALUE								(MA204_SPECIFIC_ERROR_BASE + 0)
#define		MA204_ERROR_WAIT_TIMEOUT_EXPIRED					(MA204_SPECIFIC_ERROR_BASE + 1)
#define		MA204_ERROR_RUNNING												(MA204_SPECIFIC_ERROR_BASE + 2)	
#define		MA204_ERROR_PARAMETER1										(MA204_SPECIFIC_ERROR_BASE + 4)
#define		MA204_ERROR_PARAMETER2										(MA204_SPECIFIC_ERROR_BASE + 5)
#define		MA204_ERROR_PARAMETER3										(MA204_SPECIFIC_ERROR_BASE + 6)
#define		MA204_ERROR_PARAMETER4										(MA204_SPECIFIC_ERROR_BASE + 7)
#define		MA204_ERROR_FAIL_ID_QUERY									(MA204_SPECIFIC_ERROR_BASE + 8)
#define		MA204_ERROR_APIS													(MA204_SPECIFIC_ERROR_BASE + 9)
#define		MA204_ERROR_OUT_OF_MEMORY									(MA204_SPECIFIC_ERROR_BASE + 10)
#define		MA204_SPECIFIC_ERROR_END									(MA204_SPECIFIC_ERROR_BASE + 11)
#define		MA204_SPECIFIC_ERROR_ABSOLUTE_MAX					(MA204_SPECIFIC_ERROR_BASE + 0x1FFF)


/**************************************************************************** 
*  Warning Codes
*****************************************************************************/
#define		MA204_SPECIFIC_WARNING_BASE								0x3FFC0900
#define		MA204_SPECIFIC_WARNING_END								(MA204_SPECIFIC_WARNING_BASE + 1)
#define		MA204_SPECIFIC_WARNING_ABSOLUTE_MAX				(MA204_SPECIFIC_WARNING_BASE + 0x6FF)


/**************************************************************************** 
*  M-Module ID Words
*****************************************************************************/
#define		MA204_IDENT_WORD_SYNC_CODE				0
#define		MA204_IDENT_WORD_MODULE_NUM				1
#define		MA204_IDENT_WORD_REVISION_NUM 		2
#define		MA204_IDENT_WORD_MODULE_CHAR 			3
#define		MA204_IDENT_WORD_VXI_SYNC_CODE 		16
#define		MA204_IDENT_WORD_VXI_ID 					17
#define		MA204_IDENT_WORD_VXI_DEVICE_TYPE  18

#define		MA204_VXI_MANF_ID									0xFC1
#define		MA204_VXI_MODEL_CODE							0xFE7


/**************************************************************************** 
*	MA204 Register offsets
*****************************************************************************/
#define		MA204_CONTROL_STATUS_REGISTER						0x00
#define		MA204_INTERRUPT_CONTROL_REGISTER				0x02
#define		MA204_TRIGGER_GATE_CONTROL_REGISTER			0x04
#define		MA204_CALIBRATION_REGISTER							0x06
#define		MA204_PULSE_PERIOD_LOW_REGISTER					0x08
#define		MA204_PULSE_PERIOD_MID_REGISTER					0x0A
#define		MA204_PULSE_PERIOD_HIGH_REGISTER				0x0C
#define		MA204_PULSE_WIDTH_LOW_REGISTER					0x0E
#define		MA204_PULSE_WIDTH_MID_REGISTER					0x10
#define		MA204_PULSE_WIDTH_HIGH_REGISTER					0x12
#define		MA204_PULSE_DELAY_LOW_REGISTER					0x14
#define		MA204_PULSE_DELAY_MID_REGISTER					0x16
#define		MA204_PULSE_DELAY_HIGH_REGISTER					0x18
#define		MA204_PULSE_PAIR_DELAY_LOW_REGISTER			0x1A
#define		MA204_PULSE_PAIR_DELAY_MID_REGISTER			0x1C
#define		MA204_PULSE_PAIR_DELAY_HIGH_REGISTER		0x1E
#define		MA204_OUTPUT_AMPLITUDE_LOW_REGISTER			0x20
#define		MA204_OUTPUT_AMPLITUDE_HIGH_REGISTER		0x22
#define		MA204_SLEW_RATE_REGISTER								0x24
#define		MA204_BURST_COUNT_LOW_REGISTER					0x26
#define		MA204_BURST_COUNT_HIGH_REGISTER					0x28
#define		MA204_ID_ACCESS_REGISTER								0xFE



/**************************************************************************** 
*	Bit Definitions
*****************************************************************************/
/*  Control/Status Register  */
#define		MA204_RUN_MODE_BITS											0x0006
#define		MA204_SYNC_OUT_ENABLE_BIT								0x1000
#define		MA204_SYNC_OUT_POLARITY_BIT							0x2000
#define		MA204_REFERENCE_ENABLE_BIT							0x0800
#define		MA204_REFERENCE_SELECT_BITS							0x0300
#define		MA204_RUN_BIT														0x0001
#define		MA204_OUTPUT_ENABLED_BIT								0x0010
#define		MA204_READY_BIT													0x8000	
#define		MA204_PULSE_PAIR_BIT										0x0008
#define		MA204_OUTPUT_IMPEDANCE_BIT							0x0040
#define		MA204_PULSE_POLARITY_BIT								0x0020
#define		MA204_REFERENCE_DETECED_BIT							0x0400
#define		MA204_OSCILLATOR_LOCKED_BIT							0x4000

/*  Calibration Register  */
#define		MA204_CALIBRATION_RESET_BIT							0x8000

/*  Trigger/Gate Control Register  */
#define		MA204_SYNCHRONOUS_GATING_BIT						0x8000
#define		MA204_GATE_MODE_BIT											0x4000
#define		MA204_GATE_SELECT_BITS									0x0F00
#define		MA204_TRIGGER_SELECT_BITS								0x000F
#define		MA204_MTRGSELB_BITS 										0x00C0
#define		MA204_MTRGSELA_BITS 										0x0030

/*  Interrupt Control Register  */
#define		MA204_INTERRUPT_TYPE_BIT									0x0100
#define		MA204_MASTER_INTERRUPT_ENABLE_BIT					0x8000
#define		MA204_END_OF_BURST_INTERRUPT_ENABLE_BIT		0x0001
#define		MA204_END_OF_BURST_FLAG_BIT								0x0010
#define		MA204_READY_BIT_INTERRUPT_ENABLE_BIT			0x0002
#define		MA204_READY_BIT_FLAG_BIT									0x0020



/**************************************************************************** 
*	Pulse Modes
*****************************************************************************/
#define		MA204_CONTINUOUS_CYCLE_MODE							0
#define		MA204_SINGLE_PULSE_MODE									1
#define		MA204_BURST_COUNT_MODE									2
#define		MA204_FOLLOW_TRIGGER_MODE								3

/**************************************************************************** 
*	Gate Modes 
*****************************************************************************/
#define		MA204_SYNCHRONOUS_GATE_MODE					1
#define		MA204_ASYNCHRONOUS_GATE_MODE				0


/**************************************************************************** 
*	Inputs
*****************************************************************************/
#define		MA204_INPUT_IGNORE											0x0									
#define		MA204_INPUT_FP_A_RISING									0x1									
#define		MA204_INPUT_FP_A_FALLING								0x9									
#define		MA204_INPUT_FP_B_RISING									0x2									
#define		MA204_INPUT_FP_B_FALLING								0xA									
#define		MA204_INPUT_BACKPLANE_TRIG_A_RISING			0x4									
#define		MA204_INPUT_BACKPLANE_TRIG_A_FALLING		0xC									
#define		MA204_INPUT_BACKPLANE_TRIG_B_RISING			0x5									
#define		MA204_INPUT_BACKPLANE_TRIG_B_FALLING		0xD									


/**************************************************************************** 
*	Parameter Ranges
*****************************************************************************/
#define		MA204_MIN_PULSE_WIDTH						10.0e-9
#define		MA204_MAX_PULSE_WIDTH						5.2

#define		MA204_MIN_PERIOD								20.0e-9
#define		MA204_MAX_PERIOD								5.2

#define		MA204_MIN_DELAY									0
#define		MA204_MAX_DELAY									5.2

#define		MA204_MIN_VOLTAGE								-2.0
#define		MA204_MAX_VOLTAGE								7.0


#define		MA204_MIN_SLEW_RATE							1000  	/* V/us */
#define		MA204_MAX_SLEW_RATE							2500		/* V/us */


#define		MA204_MAX_BURST_COUNT						0xFFFFFFFFl

#define		MA204_SYNCHRONOUS_GATE_MODE			1
#define		MA204_ASYNCHRONOUS_GATE_MODE		0

#define		MA204_SLEW_RATE_RESOLUTION			0.366


/**************************************************************************** 
*	Misc. Definitions
*****************************************************************************/
#define		MA204_TRUE	1
#define		MA204_FALSE	0


/**************************************************************************** 
*	Exported Function Prototypes
*****************************************************************************/

/*  Driver Configuration Functions  */
Int32 ma204_WaitForReady (MA204_HANDLE handle, Bool wait);
Int32 ma204_SetWaitTimeout (MA204_HANDLE handle, Real64 timeout_ms);

/*  Configuration Functions  */
Int32 ma204_SetPeriod (MA204_HANDLE handle, Real64 period);
Int32 ma204_SetPulseWidth (MA204_HANDLE handle, Real64 pulseWidth);
Int32 ma204_SetDelay (MA204_HANDLE handle, Real64 delay);
Int32 ma204_SetPulsePairDelay (MA204_HANDLE handle, Real64 delay);
Int32 ma204_SetAmplitude (MA204_HANDLE handle, Real64 highLevel, Real64 lowLevel);
Int32 ma204_SetRunMode (MA204_HANDLE handle, Int32 runMode);
Int32 ma204_SetPulsePair (MA204_HANDLE handle, Bool pulsePair);
Int32 ma204_SetBurstCount (MA204_HANDLE handle, UInt32 count);
Int32 ma204_SetPulsePolarity (MA204_HANDLE handle, Bool polarity);
Int32 ma204_SetPulseOutputImpedance (MA204_HANDLE handle, Bool impedance);
Int32 ma204_SetPulseOutputSlewRate (MA204_HANDLE handle, Real64 slewRate);
Int32 ma204_SetCalibrationValue (MA204_HANDLE handle, UInt16 calibrationValue);
Int32 ma204_ConfigureMTriggers (MA204_HANDLE handle, Int32 m_trigger_a, Int32 m_trigger_b);
Int32 ma204_ConfigureGateSource (MA204_HANDLE handle, Bool enable, Bool mode, Int32 select);
Int32 ma204_ConfigureTriggerSource (MA204_HANDLE handle, Int32 select);
Int32 ma204_ConfigureSyncOut (MA204_HANDLE handle, Bool enable, Bool polarity);
Int32 ma204_ConfigureExternalReference (MA204_HANDLE handle, Bool enable, Int32 select);
Int32 ma204_ConfigurePulseOutputCharacteristics (MA204_HANDLE handle, Real64 slewRate, Bool impedance,
                                                      					Bool polarity);
                                                      					
/*  Interrupt Configuration Functions  */                                                      					
Int32 ma204_ConfigIntConditions (MA204_HANDLE handle, Bool endOfBurst, Bool readyBit);

/*  Action/Status Functions  */                                       					
Int32 ma204_Run (MA204_HANDLE handle);
Int32 ma204_Stop (MA204_HANDLE handle);
Int32 ma204_OutputEnable (MA204_HANDLE handle, Bool enabled);
Int32 ma204_ExternalReferenceStatus (MA204_HANDLE handle, Bool *referenceDetected, Bool *oscillatorLocked);
Int32 ma204_Ready (MA204_HANDLE handle, Bool *ready);
Int32 ma204_EndOfBurst (MA204_HANDLE handle, Bool *endOfBurst);
Int32 ma204_ClearEndOfBurst (MA204_HANDLE handle);
Int32 ma204_CalibrationReset (MA204_HANDLE handle);

/*  Interrupt Action Status Functions  */
Int32 ma204_ClearInterrupts (MA204_HANDLE handle, Bool endOfBurst, Bool readyBit);
Int32 ma204_GetInterruptPending (MA204_HANDLE handle, Bool *endOfBurst, Bool *readyBit);

/*  Application Functions  */
Int32 ma204_CreateContinuousPulse (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                        					Real64 highLevel, Real64 lowLevel);
Int32 ma204_CreateContinuousPulsePair (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                            					Real64 highLevel, Real64 lowLevel, Real64 pulsePairDelay);
Int32 ma204_CreateSinglePulse (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                    					Real64 highLevel, Real64 lowLevel);
Int32 ma204_CreateSinglePulsePair (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                        					Real64 highLevel, Real64 lowLevel, Real64 pulsePairDelay);
Int32 ma204_CreatePulseBurst (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                   					Real64 highLevel, Real64 lowLevel, UInt32 burstCount);
Int32 ma204_CreatePulsePairBurst (MA204_HANDLE handle, Real64 pulseWidth, Real64 period, Real64 delay,
                                       					Real64 highLevel, Real64 lowLevel, Real64 pulsePairDelay, UInt32 burstCount);
Int32 ma204_CreateSquareWave (MA204_HANDLE handle, Real64 period, UInt16 dutyCycle__positive, Real64 highLevel,
                                   					Real64 lowLevel);
Int32 ma204_FollowTriggerInput (MA204_HANDLE handle, Int32 trigger, Real64 highLevel, Real64 lowLevel);

/*  Utility Functions  */
Int32 ma204_RevisionQuery (INT8 driverRev[]);
Int32 ma204_ErrorMessage (Int32 statusCode, INT8 errorMessage[]);
Int32 ma204_GetIdentWord (MA204_HANDLE handle, UInt16 offset, UInt16 *value);


/*****************************************************************************
*	Modify the functions below this point to use the MA204 driver in an 
*	environment not supported by an APIS platform support module. All functions
*	above this point should not have to be modified.
*
*	In general, any call to an APIS function (apis_func()) will need to be
*	modified to communicate with the instrument or operating system being used.
*
******************************************************************************/
Int32 ma204_Init (Int32 instr, Bool IDQuery, Bool reset, MA204_HANDLE *handle);
Int32 ma204_Close (MA204_HANDLE handle);
Int32 ma204_Reset (MA204_HANDLE handle);
Int32 ma204_ReadRegister (MA204_HANDLE handle, UInt16 offset, UInt16 *value);
Int32 ma204_WriteRegister (MA204_HANDLE handle, UInt16 offset, UInt16 value);
Int32 ma204_EnableInterrupts (MA204_HANDLE handle, Bool interruptEnable);
Int32 ma204_InstallIrq (MA204_HANDLE handle, void *interruptHandler, Int32 vector, Int32 level, Int32 mode, void *var_ptr);
Int32 ma204_UninstallIrq (MA204_HANDLE handle);
Int32 ma204_ReEnableInterrupts (MA204_HANDLE handle);
Int32 ma204_GetApisError (MA204_HANDLE handle, UInt32 *apis_error);


#endif  //MA204_HEADER

